<?php

namespace Wechat\Work;

use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Uri;

class Base
{

    protected $client;

    protected $config;
    protected $agent;

    public function __construct($config)
    {
        $this->config = $config;
        $this->client = new Client([
            'base_uri' => 'https://qyapi.weixin.qq.com/cgi-bin/',
            'timeout' => '3',
        ]);
    }

    public function agent($key)
    {
        $this->agent = $key;
        return $this;
    }

    public function getAgentId()
    {
        return $this->getAgentConfig()['id'];
    }

    public function transformForJsonRequest($uri, array $params = null)
    {
        $uri = new Uri($uri);
        $uri = Uri::withQueryValue($uri, 'access_token', $this->getAccessToken());
        if (!empty($params)) {
            $body = \GuzzleHttp\json_encode($params);
        } else {
            $body = null;
        }

        $request = new Request('POST', $uri, [], $body);
        return $request;
    }

    public function transformForGetAccessToken($uri, $params)
    {
        $uri = new Uri($uri);
        foreach ($params as $param) {
            $uri = Uri::withQueryValues($uri, $params);
        }

        $request = new Request('GET', $uri, []);
        return $request;
    }

    public function request($request)
    {
        $resp = $this->client->send($request);
        return json_decode($resp->getBody(), true);
    }

    protected function getAccessToken()
    {
        $config = $this->getAgentConfig();
        //通讯录的secret没有id
        if (!isset($config['id'])) {
            $key = 'wx:work:access_token:' . $config['key'];
        } else {
            $key = 'wx:work:access_token:' . $config['id'];
        }

        $cache = app('cache');
        $token = $cache->get($key);
        if ($token) {
            return $token;
        }

        $request = $this->transformForGetAccessToken('gettoken', [
            'corpid' => $config['corp_id'],
            'corpsecret' => $config['secret']
        ]);

        $body = $this->request($request);

        if ($body['errcode'] == 0) {
            $cache->put($key, $body['access_token'], $body['expires_in'] / 60);
            return $body['access_token'];
        } else {
            return null;
        }
    }

    protected function getAgentConfig()
    {
        if (isset($this->config['agents'][$this->agent])) {
            $config = array_merge($this->config['agents'][$this->agent], $this->config, ['key' => $this->agent]);
            unset($config['agents']);
            return $config;
        }
        return null;
    }


}