<?php

namespace Wechat\Work\Message;

abstract class MessageInterface
{
    const TYPE_TEXT = 'text';
    const TYPE_IMAGE = 'image';
    const TYPE_VOICE = 'voice';
    const TYPE_VIDEO = 'video';
    const TYPE_FILE = 'file';
    const TYPE_TEXT_CARD = 'textcard';
    const TYPE_NEWS = 'news';
    const TYPE_MP_NEWS = 'mpnews';
    const TYPE_MINI_PROGRAM_NOTICE = 'miniprogram_notice';

    protected $type;

    protected $params;
    protected $attr;

    public function toUser($user)
    {
        if (is_array($user)) {
            $user = implode('|', $user);
        }
        $this->params['touser'] = $user;
        return $this;
    }

    public function toTag($tag)
    {
        if (is_array($tag)) {
            $tag = implode('|', $tag);
        }
        $this->params['totag'] = $tag;
        return $this;
    }

    public function toArray()
    {
        $params = array_merge([
            'msgtype' => $this->type,
            'safe' => 0,
        ], $this->params);

        switch ($this->type) {
            case self::TYPE_TEXT:
                $params = array_merge($params, [
                    $this->type => [
                        'content' => $this->attr['content'],
                    ],
                ]);
                break;
            case self::TYPE_TEXT_CARD:
                $params = array_merge($params, [
                    $this->type => [
                        'title' => $this->attr['title'],
                        'description' => $this->attr['description'],
                        'url' => $this->attr['url'],
                        'btntxt' => $this->attr['btntxt'],
                    ],
                ]);
                break;
            case self::TYPE_MINI_PROGRAM_NOTICE:
                $params = array_merge($params, [
                    $this->type => $this->attr,
                ]);
                break;
            default:
                throw new \Exception('message type ' . $this->type . ' is not allow');
        }
        return $params;
    }

}